@@ -10,16 +10,20 @@ Created on Sat Oct 22 14:50:47 2022
 
 import os
 from typing import Optional, Dict, Any
+from itertools import product
 from warnings import warn
 
 import torch
 import torch.distributed as dist
-from torch.nn import Module, SyncBatchNorm, parallel
+from torch.nn import Module, SyncBatchNorm
+from torch.nn.parallel import DistributedDataParallel as DDP
 from torch.utils.data import Dataset, DataLoader
 from torch.utils.data.distributed import DistributedSampler
 
 from dstk.pytorch._utils import is_dist_avail_and_initialized
 from dstk.pytorch._base import BaseEnvironnement
+from dstk.pytorch._classifier import BaseClassifier, BaseClassifierOnline
+from dstk.pytorch._regressor import BaseRegressor, BaseRegressorOnline
 
 
 def is_main_process():
@@ -84,41 +88,69 @@ def init_distributed(dist_url: str = "env://", backend: str = "nccl"):
     return local_rank
 
 
-class DistributedDataParallel(
-    parallel.DistributedDataParallel,
-    BaseEnvironnement
-):
+def overload_distributed_data_parallel(model: Module):
     r"""
-    Classe interface à torch.nn.parallel.DistributedDataParallel. Pour
-    connaitre les arguements de cette classe ce référer à la doc PyTorch.
+    Fonction qui vien surchargé la classe
+    torch.nn.parallel.DistributedDataParallel afin de s'interfacer avec
+    les classes DSTK. Pour connaitre les arguements de cette classe ce
+    référer à la doc PyTorch.
+
+    Parameters: Module
+        Modèle PyTorch vania ou hérité des classes DSTK.
     """
-    def __init__(
-        self,
-        module,
-        device_ids=None,
-        output_device=None,
-        dim=0,
-        broadcast_buffers=True,
-        process_group=None,
-        bucket_cap_mb=25,
-        find_unused_parameters=False,
-        check_reduction=False,
-        gradient_as_bucket_view=False,
-        static_graph=False,
-    ):
-        super().__init__(
-            module=module,
-            device_ids=device_ids,
-            output_device=output_device,
-            dim=dim,
-            broadcast_buffers=broadcast_buffers,
-            process_group=process_group,
-            bucket_cap_mb=bucket_cap_mb,
-            find_unused_parameters=find_unused_parameters,
-            check_reduction=check_reduction,
-            gradient_as_bucket_view=gradient_as_bucket_view,
-            static_graph=static_graph,
+    def gen_class(module):
+        class DistributedDataParallel(DDP, module):
+            def __init__(
+                self,
+                module,
+                device_ids=None,
+                output_device=None,
+                dim=0,
+                broadcast_buffers=True,
+                process_group=None,
+                bucket_cap_mb=25,
+                find_unused_parameters=False,
+                check_reduction=False,
+                gradient_as_bucket_view=False,
+                static_graph=False,
+            ):
+                super().__init__(
+                    module=module,
+                    device_ids=device_ids,
+                    output_device=output_device,
+                    dim=dim,
+                    broadcast_buffers=broadcast_buffers,
+                    process_group=process_group,
+                    bucket_cap_mb=bucket_cap_mb,
+                    find_unused_parameters=find_unused_parameters,
+                    check_reduction=check_reduction,
+                    gradient_as_bucket_view=gradient_as_bucket_view,
+                    static_graph=static_graph,
+                )
+        return DistributedDataParallel
+
+    mother_model = model.__class__.__bases__
+    dstk_module = [BaseEnvironnement, BaseClassifier, BaseClassifierOnline,
+                   BaseRegressor, BaseRegressorOnline]
+    match = list(
+        filter(
+            lambda x: x[0] == x[1],
+            product(mother_model, dstk_module)
         )
+    )
+    if len(match) == 0:
+        return DDP
+    elif len(match) == 1:
+        return gen_class(match[0][1])
+    else:
+        match = list(filter(lambda x: x[0] != BaseEnvironnement, match))
+        if len(match) == 1:
+            return gen_class(match[0][1])
+        else:
+            raise AttributeError(
+                "L'héritage du module PyTorch est incompatible avec le "
+                "système d'hérithage de DSTK."
+            )
 
 
 def auto_init_distributed(
@@ -190,7 +222,10 @@ def auto_init_distributed(
 
     model = model.cuda()
     model = SyncBatchNorm.convert_sync_batchnorm(model)
-    model = DistributedDataParallel(model, device_ids=[local_rank])
+    model = overload_distributed_data_parallel(model)(
+        module=model,
+        device_ids=[local_rank]
+    )
     shuffle = None
     drop_last = None
     if 'shuffle' in kwargs_train_dataloader:
